//
// Program du.c. Copyright 1990 Jones Data Systems.
//

//
// Compile with:        tcc -mt -lt du.c
//

#define VERSION "v1.10"

#include <stdio.h>
#include <dir.h>
#include <dos.h>
#include <string.h>

#define UPPERCASE(x) ((((x)>='a')&&((x)<='z'))?((x)-32):(x))
#define LOWERCASE(x) ((((x)>='A')&&((x)<='Z'))?((x)+32):(x))

long bytes_per_cluster;
char summary = 0;
char no_recurse = 0;
char every = 0;

long disk_usage (char *);

void printout (long, char *);

int main (int ac, char **av)
{
    char path[70];
    int startdisk;
    int tail, loop;
    long x;
    unsigned int al, cx, ds;

    printf ("\ndu %s. Copyright 1990 Jones Data Systems.\n\n", VERSION);

    strcpy (path, ".");                     // default path == cwd

    for (loop = 1; loop < ac; loop++)
        if ((*av[loop] == '-') || (*av[loop] == '/'))
            switch (av[loop][1]) {
                case 'a':
                case 'A':
                    every = 1;
                    break;
                case 's':
                case 'S':
                    summary = 1;
                    break;
                case 'n':
                case 'N':
                    no_recurse = 1;
                    break;
                default:
                    printf ("Unrecognized flag: %s\n\n", av[loop]);
                    printf ("Valid flags:\n\t-a\tShow all files\n");
                    printf ("\t-n\tDo not recurse into subdirectories\n");
                    printf ("\t-s\tPrint summary only\n");
                    return 1;
            }
        else
            strcpy (path, av[loop]);

// terminate path with a backslash, unless it ends with a colon

    tail = strlen (path);
    if ((path[tail-1] != '\\') && (path[tail-1] != ':')) {
        path[tail] = '\\';
        path[++tail] = '\0';
    }

// Get cluster size for correct drive

    startdisk = getdisk();
    if (path[1] == ':')
        setdisk (UPPERCASE(path[0])-'A');

    ds = _DS;          // Save (and later restore) DS, as 21h/1bh messes it up
    _AH = 0x1b;
    geninterrupt (0x21);   // get cluster size
    _DS = ds;
    al = _AL;
    cx = _CX;
    bytes_per_cluster = al * cx;

    setdisk (startdisk);

    x = disk_usage (path);
    printout (x, path);

    return 0;
}

long disk_usage (char *path)
{
    struct ffblk findit;
    char ourpath[80];
    char printpath[80];
    long subtotal = 0;
    long lastreturn;

    strcpy (ourpath, path);
    strcat (ourpath, "*.*");

//
// Logic: Search for subdirectories and run disk_usage on them.
// Then search for other files and add that to the total disk_usage returned.
//

    if (!no_recurse)
        if (0 == findfirst (ourpath, &findit, FA_DIREC)) do
            if (strcmp (findit.ff_name, ".") && strcmp (findit.ff_name, "..")
                && (findit.ff_attrib & FA_DIREC)) {
                strcpy (ourpath, path);
                strcat (ourpath, findit.ff_name);
                strcat (ourpath, "\\");
                subtotal += lastreturn = disk_usage (ourpath);
                if (!summary)
                    printout (lastreturn, ourpath);
            }
        while (0 == findnext (&findit));

    strcpy (ourpath, path);
    strcat (ourpath, "*.*");

    if (0 == findfirst (ourpath, &findit, FA_ARCH+FA_DIREC+FA_LABEL+FA_SYSTEM+FA_HIDDEN+FA_RDONLY))
    do {
        subtotal += ((findit.ff_fsize / bytes_per_cluster) +
                     (findit.ff_fsize % bytes_per_cluster != 0))
                    * bytes_per_cluster
                    / 512;
        if (every)
            if (findit.ff_name[strlen(findit.ff_name)-1] != '.') {
                strcpy (printpath, path);
                strcat (printpath, findit.ff_name);
                strcat (printpath, "\\");
                printout (((findit.ff_fsize / bytes_per_cluster) +
                          (findit.ff_fsize % bytes_per_cluster != 0))
                           * bytes_per_cluster / 512, printpath);
            }
    } while (0 == findnext (&findit));

    return (subtotal);
}

void printout (long size, char *path)
{
    char ourpath[80];
    char *pathptr1 = &ourpath[0];
    char *pathptr2 = path;

    while (*pathptr2) {
        *pathptr1 = LOWERCASE (*pathptr2);
        pathptr2++;
        if (*pathptr2 == '\0')
            *pathptr1 = '\0';
        pathptr1++;
    }

    if ((*ourpath == 0) || (ourpath[1] == ':'))
        strcat (ourpath, "\\");

    if (path[strlen(path)-1] == ':')
        strcat (ourpath, ":");

//  #define PATTERN "%ld\t%-10ld  %s\n"

//  printf (PATTERN, size, size * 512, ourpath);
    printf ("%ld\t%s\n", size, ourpath);
}

